
from file_utility import remove_temp_file, valid_file, valid_directory
from keras import Model


def show_summary(model):
    model.summary()


def show_summary_model(path_model):
    print("show summary of keras saved model...")
    model = load_from_saved_model(path_model)
    show_summary(model)


def show_summary_weights(path_topology, path_weights):
    print("show summary of keras saved topology + weights...")
    model = load_from_saved_weights(path_topology, path_weights)
    show_summary(model)


def preprocess_from_model(path_model, path_output_dir, output_node_names):
    if (not valid_file(path_model)):
        print('Aboard converting... INVALID input file.')
        return
    if (not valid_directory(path_output_dir)):
        print('Aboard converting... INVALID output directory.')
        return

    model = load_from_saved_model(path_model)
    enc_model: Model = generate_encapsulate_model_with_output_layer_names(model, split_layer_name_list(output_node_names))

    # Generate temp Keras enc_model for further processing
    print("Saving enc_model...")
    save_enc_model(path_output_dir, enc_model)
    print("Saving converted tfjs model...")
    convert_tfjs(path_output_dir)
    print("Remove enc_model file...")
    remove_temp_file(path_output_dir+"/enc_model.h5")
    print("Mission Complete!!!")


def load_from_saved_model(path_model):
    from keras.models import load_model
    model = load_model(path_model)
    return model


def load_from_saved_weights(path_topology, path_weights):
    from keras.models import model_from_json
    structure = open(path_topology, "r")
    model = model_from_json(structure)
    model.load_weights(path_weights)
    return model


def generate_encapsulate_model_with_output_layer_names(model, output_layer_names):
    from keras.models import Model
    enc_model = Model(
        inputs=model.input,
        outputs=list(map(lambda oln: model.get_layer(oln).output, output_layer_names))
    )
    return enc_model


def split_layer_name_list(output_node_names):
    return output_node_names.split("/")


def save_enc_model(path_output_dir, enc_model):
    from keras.models import save_model
    save_model(enc_model, path_output_dir+"/enc_model.h5")


def convert_tfjs(path_output_dir):
    from tensorflowjs.converters.converter import dispatch_keras_h5_to_tensorflowjs_conversion
    dispatch_keras_h5_to_tensorflowjs_conversion(
        path_output_dir+"/enc_model.h5",
        path_output_dir
    )



import argparse


from keras_conversion import show_summary_model, preprocess_from_model
from version import version


def print_hello_world():
    print("Hello World from converter")


def main():
    parser = argparse.ArgumentParser('TensorSpace.js model converter/preprocessor.')
    parser.add_argument(
        'input_path',
        nargs='?',
        type=str,
        help='Path to the input file or directory. For input format "keras", '
             'an HDF5 (.h5) file or structure (.json) and weight (.hdf5) directory '
             'is expected.'
    )
    parser.add_argument(
        'output_path', nargs='?', type=str, help='Path for all output artifacts.'
    )
    parser.add_argument(
        '--input_type',
        type=str,
        required=False,
        default='keras',
        choices=set(['keras']),
        help='Input model type.\n'
             'It could be "keras", ... for now.'
    )
    parser.add_argument(
        '--input_format',
        type=str,
        required=False,
        default='keras_saved_model',
        choices=set(['keras_saved_model', 'keras_saved_weight']),
        help='Input format.\n'
             'For "keras_saved_model", input is .h5 saved by .save().\n'
             'For "keras_saved_weight", inputs are topology+weights.'
    )
    parser.add_argument(
        '--output_node_names',
        type=str,
        help='The names of the output nodes, separated by slash. '
             'E.g., "logits/activations".')
    parser.add_argument(
        '--version',
        '-v',
        dest='show_version',
        action='store_true',
        help='Show versions of tensorspacejs and its dependencies'
    )
    parser.add_argument(
        '--summary',
        '-s',
        dest='show_model_summary',
        action='store_true',
        help='Show summray of loaded model'
    )

    flags = parser.parse_args()

    if flags.show_version:
        print('\ntensorspacejs %s\n' % version)
        print('Dependency versions:')
        print('(TBD)')
        # print('  keras %s' % keras.__version__)
        # print('  tensorflow %s' % tf.__version__)
        return

    if flags.input_path is None:
        raise ValueError(
            'Error: The input_path argument must be set. '
            'Run with --help flag for usage information.')

    if flags.input_type not in ('keras'):
        raise ValueError(
            'The --input_type flag can only be set to '
            '"keras" '
            'but the current input type is "%s".' % flags.input_type)

    if flags.input_type == 'keras' and flags.input_format not in ('keras_saved_model', 'keras_saved_weight'):
        raise ValueError(
            'For input_type == "keras", the --input_format flag can only be set to'
            '"keras_saved_model" and "keras_saved_weight" '
            'but the current input format is "%s".' % flags.input_format)

    if flags.show_model_summary:
        if flags.input_type == 'keras':
            show_summary_model(flags.input_path)
            return

    if flags.input_type == 'keras' and flags.input_format == 'keras_saved_model':
        preprocess_from_model(flags.input_path, flags.output_path, flags.output_node_names)
        return

    print("Nothing happened...")


if __name__ == '__main__':
    main()


import subprocess

def main():
    # subprocess.call(["ls", "-l"])
    # subprocess.call(["echo", "Hello World~"])
    subprocess.check_call(["echo", "Hello World~"])
    subprocess.check_call(["pwd"])

    main_js_path = "/Users/zchholmes/PycharmProjects/tensorspacejs/src/tfjs/main.js"
    output_names = "--output_layer_names='myPadding/myConv1/myMaxPooling1/myConv2/myMaxPooling2/myDense1/myDense2/myDense3'"
    path_input = "/Users/zchholmes/PycharmProjects/tensorspacejs/src/tfjs/test/sequential/mnist.json"
    path_output = "/Users/zchholmes/PycharmProjects/tensorspacejs/src/tfjs/test"
    subprocess.check_call(["node", main_js_path, output_names, path_input, path_output])



if __name__ == '__main__':
    main()